
function main(...)

	local lfs = require("lfs")
	local iconv = require("iconv")
	local utf8_to_gbk = iconv.new("GBK", "UTF-8")
	local gbk_to_utf8 = iconv.new("UTF-8", "GBK")

	local path = ...
	if not path then
		print("Need path.")
		os.execute("pause")
		return
	end

	local vflist = {}

	local function _check_path(path, mname_plus, path_plus, current_type)
		if mname_plus then
			mname_plus = mname_plus.."."
		else
			mname_plus = ""
		end
		if path_plus then
			path_plus = path_plus.."/"
		else
			path_plus = ""
		end
		for filename in lfs.dir(path) do
			if (filename ~= ".." and filename ~= ".") then
				print(path_plus..filename)
				local file_attr = lfs.attributes(path.."\\"..filename)
				if getextension(filename)=="lua" and file_attr.mode=="file" then
					local data = freads(path.."\\"..filename)
					local func, syntax_error = load(data)
					if func then
						local mname = stripextension(filename)
						local info = {
							type = "lua",
							name = gbk_to_utf8:iconv(mname_plus..mname),
							data = data,
						}
						if mname=="main" then
							vflist.main = info
						else
							vflist[#vflist + 1] = info
						end
					else
						print(syntax_error)
						os.execute("pause")
						error("interrupt")
					end
				elseif getextension(filename)=="xxt" and file_attr.mode=="file" then
					local data = s2h(freads(path.."\\"..filename))
					vflist[#vflist + 1] = {
						type = current_type or "xxt",
						name = gbk_to_utf8:iconv(path_plus..filename),
						data = data,
					}
				else
					if file_attr.mode=="file" then
						local data = s2h(freads(path.."\\"..filename))
						vflist[#vflist + 1] = {
							type = current_type or "other",
							name = gbk_to_utf8:iconv(path_plus..filename),
							data = data,
						}
					elseif file_attr.mode=="directory" then
						local sub_current_type = current_type
						if mname_plus=="" then
							if filename=="res" or filename=="lib" then
								sub_current_type = sub_current_type or filename
							end
						end
						_check_path(path.."\\"..filename, mname_plus..filename, path_plus..filename, sub_current_type)
					end
		    	end
			end
		end
	end

	local path_attr = lfs.attributes(path)
	if path_attr.mode=="directory" then
		print("Checking...")
		_check_path(path)
	else
		print("`"..path.."` is not a directory.")
		os.execute("pause")
		error("interrupt")
	end

	print("Merging...")

	local out_tab = {}
	local type_root_map = {
		other = [["/var/mobile/Media/1ferver/lua/scripts/"]],
		xxt = [["/var/mobile/Media/1ferver/lua/"]],
		lib = [["/var/mobile/Media/1ferver/"]],
		res = [["/var/mobile/Media/1ferver/"]],
	}

	out_tab[#out_tab + 1] = [[
do
local lfs = require "lfs"
lfs.chdir("/var/mobile/Media/1ferver/lua/scripts/")
local function sh_escape(dir)
	dir = string.gsub(dir, "([ \\()<>'\"`#&*;?~$])", "\\%1")
	return dir
end
local function stripfilename(filename)
	return string.match(filename, "^(.*/)[^/]+$") -- *nix system
end
local dir, fn
]]
	for _,info in ipairs(vflist) do
		if info.type ~= "lua" then
			out_tab[#out_tab + 1] = string.format([[
fn = %s..%q
dir = stripfilename(fn)
io.popen('mkdir -p '..sh_escape(dir)):read('*a')
file.writes(fn, "%s")
io.popen('chown mobile:mobile '..sh_escape(dir)..';chown mobile:mobile '..sh_escape(fn)):read('*a')
]], type_root_map[info.type], info.name, info.data)
		end
	end
	out_tab[#out_tab + 1] = "end"

	for _,info in ipairs(vflist) do
		if info.type == "lua" then
			out_tab[#out_tab + 1] = string.format([[
package.preload[%q] = function(...)
%s
end
]], info.name, info.data)
		end
	end

	if vflist.main then
		out_tab[#out_tab + 1] = vflist.main.data
	end

	local merged_src = table.concat(out_tab, "\n")

	fwrites("bin/xxt_packer", merged_src)

	if fexists("bin/xxt_packer") then
		if fexists(path..".lua") then
			os.remove(path..".lua.bak")
			local success, err = os.rename(path..".lua", path..".lua.bak")
			if not success then
				print(err)
				os.execute("pause")
				error("interrupt")
			end
		end
		print("Done...")
		os.rename("bin/xxt_packer", path..".lua")
	end

end






























function s2h(str,spacer)return (string.gsub(str,"(.)",function (c)return string.format("\\x%02x%s",string.byte(c), spacer or"")end))end

function getextension(filename)
	return filename:match("^.*%.(%w+)$") or ''
end

function stripextension(filename)
	local idx = filename:match(".+()%.%w+$")
	if(idx) then
		return filename:sub(1, idx-1)
	else
		return filename
	end
end

function stripfilename(filename)
	return string.match(filename, "^(.+\\)[^\\]+$") -- Windows
end

function strippath(filename)
	return string.match(filename, "^.+\\([^\\]+)$") or "" -- Windows
end

function freads(filename)
	local f = io.open(filename, "rb")
	if (f) then
		local s = f:read("*a")
		f:close()
		return s
	end
	return ""
end

function fexists(filename)
	local f = io.open(filename, "rb")
	if (f) then
		f:close()
		return true
	else
		return false
	end
end

function fwrites(filename, contents)
	local f = io.open(filename, "wb")
	if (f) then
		f:write(contents)
		f:close()
		return true
	end
	return false
end

function popen(cmd)
	os.remove("popen_stdout")
	os.remove("popen_stderr")
	os.execute(cmd.." 1>popen_stdout 2>popen_stderr")
	local out, err = freads("popen_stdout"), freads("popen_stderr")
	os.remove("popen_stdout")
	os.remove("popen_stderr")
	return out, err
end

return main(...)