--[[

	string_ex.lua

	Created by 苏泽 on 17-08-20.
	Copyright (c) 2017年 苏泽. All rights reserved.

--]]

local openssl = require('openssl')
local iconv = require('iconv')

local gbk_to_utf8 = iconv.new("UTF-8", "GBK")

function string.from_gbk(self)
	return gbk_to_utf8(self)
end

function string.aes128_encrypt(self, key)
	local evp_cipher = openssl.cipher.get('aes-128-ecb')
	return evp_cipher:encrypt(self, key)
end

function string.aes128_decrypt(self, key)
	local evp_cipher = openssl.cipher.get('aes-128-ecb')
	return evp_cipher:decrypt(self, key)
end

function string.to_hex(self)
	return openssl.hex(self, true)
end
string.tohex = string.to_hex

function string.from_hex(self)
	return openssl.hex(self, false)
end
string.fromhex = string.from_hex

function string.base64_encode(self)
	return openssl.base64(self, true, false)
end

function string.base64_decode(self)
	return openssl.base64(self, false, false)
end

function string.md5b(self)
	return openssl.digest.get('md5').digest(self)
end

function string.md5(self)
	return openssl.hex(openssl.digest.get('md5'):digest(self), true)
end

function string.sha1b(self)
	return openssl.digest.get('sha1').digest(self)
end

function string.sha1(self)
	return openssl.hex(openssl.digest.get('sha1'):digest(self), true)
end

function string.hmac_sha1b(self, key)
	return openssl.hmac.hmac('sha1', self, key, true)
end

function string.hmac_sha1(self, key)
	return openssl.hmac.hmac('sha1', self, key, false)
end

function string.hmac_sha256b(self, key)
	return openssl.hmac.hmac('sha256', self, key, true)
end

function string.hmac_sha256(self, key)
	return openssl.hmac.hmac('sha256', self, key, false)
end

function string.split(self, sep)
	local str = self
	local strtab = {}
	while (true) do
		local pos = str:find(sep, 1, true)
		if (not pos) then
			strtab[#strtab + 1] = str
			break
		end
		strtab[#strtab + 1] = str:sub(1, pos - 1)
		str = str:sub(pos + #sep, #str)
	end
	return strtab
end

function string.ltrim(self)
	return (string.gsub(self, "^%s*(.-)", "%1"))
end

function string.rtrim(self)
	return (string.gsub(self, "(.-)%s*$", "%1"))
end

function string.trim(self)
	return (string.gsub(self, "^%s*(.-)%s*$", "%1"))
end

function string.atrim(self)
	return (string.gsub(self, "%s", ""))
end

function string.random(self, char_count, char_len)
	if type(self)~='string' then
		error(string.format("bad argument #1 to 'string.random' (non-empty string expected got %s)", type(self)), 2)
	end
	if #self==0 then
		error("bad argument #1 to 'string.random' (non-empty string expected got '')", 2)
	end
	char_count = char_count or 6
	char_len = char_len or 1
	local in_tab = {}
	local sub = string.sub
	local ran = math.random
	local concat = table.concat
	for i=1,#self do
		in_tab[#in_tab + 1] = sub(self, i, i)
	end
	local out_tab = {}
	for i=1,char_count do
		local n = ran(1, (#self)/char_len)
		n = 1 + (n - 1) * char_len
		out_tab[#out_tab + 1] = concat(in_tab, "", n, n + (char_len - 1))
	end
	return concat(out_tab)
end


-------------------------------------------------------------------------


-- local utf8_to_gbk = iconv.new("GBK", "UTF-8")
-- sys = {
-- 	alert = function(txt)
-- 		print(utf8_to_gbk:iconv(txt))
-- 	end,
-- }

-- -- 哈希校验
-- local str = "sozereal"
-- sys.alert('"'..str..'" 的 16 进制编码为: <'..str:to_hex()..'>')
-- sys.alert('<'..str:to_hex()..'> 转换成明文为: "'..str:to_hex():from_hex()..'"')
-- sys.alert('"'..str..'" 的 MD5 值是: '..str:md5())
-- sys.alert('"'..str..'" 的 SHA1 值是: '..str:sha1())
-- local binstr = "\0\1\2\3\4\5"
-- sys.alert('<'..binstr:to_hex()..'> 的 MD5 值是: '..binstr:md5())
-- sys.alert('<'..binstr:to_hex()..'> 的 SHA1 值是: '..binstr:sha1())
-- -- 数据加/解密
-- local msg = "\5\4\3\2\1\0"
-- local key = "sozereal"
-- local emsg = msg:aes128_encrypt(key)
-- local emsgb64 = emsg:base64_encode()
-- sys.alert('二进制数据<'..msg:to_hex()..'> \n 使用 AES128 算法 密钥 "'..key..'" 加密 值是: <'..emsg:to_hex()..'> \n base64 串为 "'..emsgb64..'"')
-- local tmp = emsgb64:base64_decode()
-- msg = tmp:aes128_decrypt(key)
-- sys.alert('"'..emsgb64..'" base64 解码后的数据为 <'..tmp:to_hex()..'> \n使用 AES128 算法 密钥 "'..key..'" 解密 值是: <'..msg:to_hex()..'>')
-- -- 字符串小工具
-- str = "  哈哈,he he,1,3,6  "
-- new = str:split(",") --将字符串str按照 `,` 分割并返回一个表
-- sys.alert(new[2])
-- sys.alert(str:rtrim()) -- 结果 "  哈哈,he he,1,3,6" ,删除字符串尾部的空白字符
-- sys.alert(str:ltrim()) -- 结果 "哈哈,he he,1,3,6  " ,删除字符串首部的空白字符
-- sys.alert(str:trim()) -- 结果 "哈哈,he he,1,3,6"    ,删除字符串首尾的空白字符
-- sys.alert(str:atrim()) -- 结果 "哈哈,hehe,1,3,6"    ,删除字符串所有的空白字符

-- local data = 'hello world'
-- local edata = data:aes128_encrypt('havonz')

-- print("<"..edata:to_hex()..">")
-- print("<"..edata:hmac_sha1('havonz')..">")

-- print(edata:aes128_decrypt('havonz'))